#include "triangle_mesh_type.h"

using namespace vcg;


  void SelectRandomVertices(MyTriMesh &m, std::vector<MyTriFace::VertexType *> &choosenVertVec)
  {
  // Select a random vertex and accept it only if all the vertices in 1-ring around it are not marked. 
  // Then mark all the vertex around it 
  // We assume that the mesh is compact (no holes in the vertex vector)

  // Clean the state of the mesh (color and visited flag)
  tri::UpdateFlags<MyTriMesh>::VertexClearV(m);
  tri::UpdateColor<MyTriMesh>::PerFaceConstant(m,Color4b::White);
  int cnt=0;
  for(int i=0;i<10000;++i)
  {
    int vi=rand()%m.VN();
    bool ok=true;
    std::vector<MyTriFace::VertexType *> VVstar;
    face::VVStarVF<MyTriFace>(&(m.vert[vi]), VVstar);

    for(size_t j=0;j<VVstar.size();++j)
      if(VVstar[j]->IsV()) ok=false;

    if(ok)
    {
      for(size_t j=0;j<VVstar.size();++j)
        VVstar[j]->SetV();        
      cnt++;
      choosenVertVec.push_back(&m.vert[vi]);
      m.vert[vi].C()=Color4b::Red;
      std::vector<MyTriMesh::FacePointer> vfp;
      std::vector<int> vfi;
      face::VFStarVF(&(m.vert[vi]),vfp,vfi);

      for(size_t j=0;j<vfp.size();++j)
        vfp[j]->C()=Color4b::Red;
    }
  } 
  printf("Selected %i vertices \n",cnt);
  }

  void ExtrudeSelectedVertices(MyTriMesh &m, std::vector<MyTriFace::VertexType *> &choosenVertVec, 
  float extrude_thickness)
  {
  float extrude_scale=1.0f;
  // let compute the bounding box of the mesh to scale the extrusion
  tri::UpdateBounding<MyTriMesh>::Box(m);
  extrude_scale=extrude_thickness*Distance(m.bbox.min,m.bbox.max);

  // Create a new mesh composed by the chosen vertices and the faces that connect them translated by 1 unit along the normal
  
  MyTriMesh m2;
  int curVertSize=0;
  int curFaceSize=0;
    // Let use a map to keep track of the correspondence between the old and the new vertexes
    std::map<MyTriVertex *, int> old2newTop;
    std::map<MyTriVertex *, int> old2newBottom;
    
  for(int i=0;i<choosenVertVec.size();++i)
  {
    MyTriVertex *vp = choosenVertVec[i];
    std::vector<MyTriFace::VertexType *> VVstar;
    face::VVStarVF<MyTriFace>(vp, VVstar);
    // add the vertexes to the new mesh
    tri::Allocator<MyTriMesh>::AddVertices(m2,VVstar.size()+1);
  
    m2.vert[curVertSize].P()=vp->P()+extrude_scale*vp->N();
    m2.vert[curVertSize].C()=Color4b::White;
    
    old2newTop[vp]=curVertSize;
    curVertSize++;
    for(size_t j=0;j<VVstar.size();++j)
    {
      m2.vert[curVertSize+j].P()=VVstar[j]->P()+extrude_scale*vp->N();
      m2.vert[curVertSize+j].C()=Color4b::White;
      old2newTop[VVstar[j]]=curVertSize+j;
    }
    curVertSize+=VVstar.size();
    
    std::vector<MyTriMesh::FacePointer> VFstar_fp;
    std::vector<int> VFstar_fi;
    face::VFStarVF(vp,VFstar_fp,VFstar_fi);

    // add the faces to the new mesh
    tri::Allocator<MyTriMesh>::AddFaces(m2,VFstar_fp.size());

    for(size_t j=0;j<VFstar_fp.size();++j)
    {
      MyTriFace &f=m2.face[curFaceSize+j];
      f.V(0)=&m2.vert[old2newTop[VFstar_fp[j]->V0(VFstar_fi[j])]];
      f.V(1)=&m2.vert[old2newTop[VFstar_fp[j]->V1(VFstar_fi[j])]];
      f.V(2)=&m2.vert[old2newTop[VFstar_fp[j]->V2(VFstar_fi[j])]];
    }
    curFaceSize+=VFstar_fp.size();
  }
  // Now copy all the not red faces onto the new mesh
  for(auto fi=m.face.begin();fi!=m.face.end();++fi)
  {
    if(fi->C()!=Color4b(Color4b::Red))
    {
      // first we find the new vertexes in the new mesh allocating it if needed
      for(int i=0;i<3;++i)
      {
        if(old2newBottom.find(fi->V(i))==old2newBottom.end())
        {
          tri::Allocator<MyTriMesh>::AddVertices(m2,1);
          m2.vert[curVertSize].P()=fi->V(i)->P();
          m2.vert[curVertSize].C()=Color4b::LightGray;
          old2newBottom[fi->V(i)]=curVertSize;
          curVertSize++;
        }
      }
      // Now add the new face
      tri::Allocator<MyTriMesh>::AddFace(m2,&(m2.vert[old2newBottom[fi->V(0)]]),
                                         &(m2.vert[old2newBottom[fi->V(1)]]),
                                         &(m2.vert[old2newBottom[fi->V(2)]]));
    }
  }

  // final pass to add the side of the extruded faces

  for(int i=0;i<choosenVertVec.size();++i)
  {
    MyTriVertex *vp = choosenVertVec[i];
    std::vector<MyTriMesh::FacePointer> VFstar_fp;
    std::vector<int> VFstar_fi;
    face::VFStarVF(vp,VFstar_fp,VFstar_fi);

    for(size_t j=0;j<VFstar_fp.size();++j)
    {
      MyTriFace *fi = VFstar_fp[j];
      int vind = VFstar_fi[j]; // the index of the central vertex of the star in this triangle
    
      tri::Allocator<MyTriMesh>::AddFace(m2,
                                        &(m2.vert[old2newTop   [fi->V1(vind)]]),
                                        &(m2.vert[old2newBottom[fi->V1(vind)]]),
                                        &(m2.vert[old2newBottom[fi->V2(vind)]]));
      tri::Allocator<MyTriMesh>::AddFace(m2,
                                        &(m2.vert[old2newBottom[fi->V2(vind)]]),
                                        &(m2.vert[old2newTop   [fi->V2(vind)]]),
                                        &(m2.vert[old2newTop   [fi->V1(vind)]]));

      m2.vert[old2newBottom[fi->V1(vind)]].C()=Color4b::Gray;
      m2.vert[old2newBottom[fi->V2(vind)]].C()=Color4b::Gray;
    }
  }

  // Now we can replace the old mesh with the new one
  tri::Append<MyTriMesh,MyTriMesh>::MeshCopy(m,m2);
  tri::UpdateTopology<MyTriMesh>::VertexFace(m);
  tri::UpdateNormal<MyTriMesh>::PerVertexNormalizedPerFace(m);
  tri::UpdateBounding<MyTriMesh>::Box(m);
  choosenVertVec.clear();
}
